﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Runtime.Serialization.Json;
using System.IO;
using System.Text;
using System.Web.Script.Serialization;
using Newtonsoft.Json;
using System.Text.RegularExpressions;
using System.Diagnostics;

//=====================================================
//Copyright (C)   http://www.cnblogs.com/luge
//All rights reserved
//文件名:           JsonFormat
//创建时间:         2015-06-21
//当前登录用户名:   卤鸽
//描述：             
//======================================================

namespace JsonConvertSample.Common
{
    /// <summary>
    /// json格式化类
    /// </summary>
    public class JsonFormat
    {
        /// <summary>
        /// Date(12345+800)正则表达式匹配
        /// </summary>
        public static  string DATE_TO_ISO_PATTERN = @"\\/Date\((\d+)([-+])(\d+)\)\\/";
        /// <summary>
        ///2015-06-21T22:09:00正则表达式匹配
        /// </summary>
        public static string ISO_TO_DATE_PATTERN = @"(\d{4})-(\d{2})-(\d{2})T(\d{2}):(\d{2}):(\d{2}(?:\.\d*)?)(?:([\+-])(\d{2})\:(\d{2}))?Z?";


        #region 辅助方法

        /// <summary>  
        /// 将Json序列化的时间由/Date(1294499956278+0800)转为字符串  
        /// </summary>  
        private static string ConvertJsonDateToDateString(Match m)
        {
            string result = string.Empty;
            DateTime dt = new DateTime(1970, 1, 1);
            dt = dt.AddMilliseconds(long.Parse(m.Groups[1].Value));
            dt = dt.ToLocalTime();
            result = dt.ToString("yyyy-MM-dd'T'HH:mm:ss");
            return result;
        }
        /// <summary>  
        /// 将时间字符串转为Json时间  
        /// </summary>  
        private static string ConvertDateStringToJsonDate(Match m)
        {
            string result = string.Empty;
            DateTime dt = DateTime.Parse(m.Groups[0].Value);
            dt = dt.ToUniversalTime();
            
            TimeSpan ts = dt - DateTime.Parse("1970-01-01");
            result = string.Format("\\/Date({0}+0800)\\/",ts.TotalMilliseconds.ToString("f0"));
            return result;
        }

        #endregion

        #region DataContractJsonSerializer

        /// <summary>
        /// 序列化json字符串
        /// </summary>
        /// <typeparam name="T"></typeparam>
        /// <param name="obj"></param>
        /// <param name="datetimePattern"> 替换Json的Date字符串  </param>
        /// <returns></returns>
        public static string SerializeByContract<T>(T obj, string datetimePattern = null)
            where T : class
        {
            string jsonString = string.Empty;

            if (obj == null)
                return jsonString;

            var ser = new DataContractJsonSerializer(typeof(T));
            using (var ms = new MemoryStream())
            {
                ser.WriteObject(ms, obj);
                jsonString = Encoding.UTF8.GetString(ms.ToArray());
            }

            if (!string.IsNullOrEmpty(datetimePattern))
            {
               
                MatchEvaluator matchEvaluator = new MatchEvaluator(ConvertJsonDateToDateString);
                Regex reg = new Regex(datetimePattern);
                jsonString = reg.Replace(jsonString, matchEvaluator);
            }

            return jsonString;
        }

        /// <summary>
        /// json反序列化成对象
        /// </summary>
        /// <typeparam name="T"></typeparam>
        /// <param name="jsonString"></param>
        /// <param name="datetimePattern">匹配时间的正则表达式</param>
        /// <returns></returns>
        public static T DeserializeByContract<T>(string jsonString, string datetimePattern = null)
            where T : class
        {
            if (string.IsNullOrEmpty(jsonString))
            {
                return default(T);
            }


            if (!string.IsNullOrEmpty(datetimePattern))
            {
                MatchEvaluator matchEvaluator = new MatchEvaluator(ConvertDateStringToJsonDate);
                var reg = new Regex(datetimePattern);
                jsonString = reg.Replace(jsonString, matchEvaluator);
            }
            
            var ser = new DataContractJsonSerializer(typeof(T));
            var jsonByteArr = Encoding.UTF8.GetBytes(jsonString);
            using (var ms = new MemoryStream(jsonByteArr))
            {
                object obj = ser.ReadObject(ms);
                return obj as T;
            }

        }

        #endregion

        #region JavaScriptSerializer

        public static string SerializeByJavaScript<T>(T obj)
            where T : class
        {
            var jsonString = string.Empty;

            if (obj == null)
            {
                return jsonString;
            }

            JavaScriptSerializer js = new JavaScriptSerializer();
            //<!--进行序列化或反序列化时出错。字符串的长度超过了为 maxJsonLength 属性设置的值-->
            js.MaxJsonLength = int.MaxValue;
            jsonString = js.Serialize(obj);

            return jsonString;
        }


        public static T DeserializeByJavaScript<T>(string jsonString)
            where T : class
        {
            T obj;
            if (string.IsNullOrEmpty(jsonString))
            {
                obj = null;
                return obj;
            }

            JavaScriptSerializer js = new JavaScriptSerializer();
            js.MaxJsonLength = int.MaxValue;
            obj = js.Deserialize<T>(jsonString);

            return obj;
        }
        #endregion
        
        #region Json.Net

        public static string SerializeByJsonNet<T>(T obj)
            where T : class
        {
            var jsonString = string.Empty;


            if (obj == null)
                return jsonString;

            jsonString = JsonConvert.SerializeObject(obj);

            return jsonString;

        }


        public static T DeserializeByJsonNet<T>(string jsonString)
            where T : class
        {
            T obj = null;
            if (string.IsNullOrEmpty(jsonString))
            {
                return obj;
            }

            obj = JsonConvert.DeserializeObject<T>(jsonString);
            return obj;
        }


        #endregion
    }
}